//
//  GLXViewController.m
//  GLES2.0 Sample
//
//  Created by Satoshi Numata on 12/04/21.
//  Copyright (c) 2012 Sazameki and Satoshi Numata, Ph.D. All rights reserved.
//

#import "GLXViewController.h"
#include <execinfo.h>
#include <iostream>


static vec2 sScreenSize;
GLKView  *_gGLKView;


@interface GLXViewController () {
}
@property (strong, nonatomic) EAGLContext *context;

- (void)setupGL;
- (void)tearDownGL;

@end

@implementation GLXViewController

@synthesize context = _context;

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];

    if (!self.context) {
        NSLog(@"Failed to create ES context");
    }
    
    GLKView *view = (GLKView *)self.view;
    view.context = self.context;
    view.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    
    _gGLKView = view;
    
    [self setupGL];
}

- (void)viewDidUnload
{    
    [super viewDidUnload];
    
    [self tearDownGL];
    
    if ([EAGLContext currentContext] == self.context) {
        [EAGLContext setCurrentContext:nil];
    }
	self.context = nil;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc. that aren't in use.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (void)setupGL
{
    [EAGLContext setCurrentContext:self.context];
    
    try {
        CGSize viewSize = self.view.bounds.size;
        CGFloat scale = self.view.contentScaleFactor;
        viewSize.width *= scale;
        viewSize.height *= scale;
        sScreenSize = vec2(viewSize.width, viewSize.height);
        mGameMain = new GameMain(sScreenSize);
    } catch (std::exception& e) {
        NSLog(@"[Error] GameMain::GameMain() => %s", e.what());
        exit(100);
    }
}

- (void)tearDownGL
{
    [EAGLContext setCurrentContext:self.context];
    
    try {
        delete mGameMain;
        mGameMain = 0;
    } catch (std::exception& e) {
        NSLog(@"[Error] GameMain::~GameMain() => %s", e.what());
        exit(104);
    }
}

#pragma mark - GLKView and GLKViewController delegate methods

- (void)update
{
    try {
        mGameMain->updateModel();
    } catch (std::exception& e) {
        NSLog(@"[Error] GameMain::updateModel() => %s", e.what());
        exit(102);
    }
}

- (void)glkView:(GLKView *)view drawInRect:(CGRect)rect
{
    try {
        mGameMain->drawView(sScreenSize);
    } catch (std::exception& e) {
        NSLog(@"[Error] GameMain::drawView() => %s", e.what());
        exit(103);
    }
}

@end

